#! /usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Replacement (unofficial) `setup.py` for the `web` module.
Automatically generated by `openerpdist`. See http://noteed.com/openerpdist/.
"""

import setuptools

setuptools.setup(
    name = "openerp-web",
    version = "7.0.406",
    description = "Web",
    long_description = 
"""

OpenERP Web core module.
========================

This module provides the core of the OpenERP Web Client.
        
""",
    url = "",
    author = "",
    author_email = "TODO",
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Environment :: Plugins",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Natural Language :: English",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    license = "AGPL-3",
    package_dir = {
        'openerp.addons.web': ".",
    },
    packages = ["openerp.addons.web","openerp.addons.web.controllers","openerp.addons.web.cli","openerp.addons.web.tests","openerp.addons.web.tests.qunitsuite"],
    package_data = {
        'openerp.addons.web': ["i18n/*.po*","static/test/testing.js","static/test/list-editable.js","static/test/list-utils.js","static/test/mutex.js","static/test/fulltest.html","static/test/qunit-doc.js","static/test/data.js","static/test/evals.js","static/test/list.js","static/test/form.js","static/test/Widget.js","static/test/formats.js","static/test/registry.js","static/test/class.js","static/test/rpc.js","static/test/search.js","static/test/fulltest/dataset.js","static/lib/jquery.placeholder/jquery.placeholder.js","static/lib/jquery.ba-bbq/jquery.ba-bbq.min.js","static/lib/jquery.ba-bbq/jquery.ba-bbq.js","static/lib/jquery.ui.notify/js/jquery.notify.js","static/lib/jquery.form/jquery.form.js","static/lib/jquery.ui.timepicker/js/jquery-ui-timepicker-addon.js","static/lib/jquery.autosize/jquery.autosize.js","static/lib/cleditor/jquery.cleditor.min.js","static/lib/cleditor/jquery.cleditor.js","static/lib/cleditor/images/toolbar.gif","static/lib/cleditor/images/buttons.gif","static/lib/datejs/parser.js","static/lib/datejs/date.js","static/lib/datejs/sugarpak.js","static/lib/datejs/time.js","static/lib/datejs/core.js","static/lib/datejs/extras.js","static/lib/datejs/globalization/zh-CN.js","static/lib/datejs/globalization/bg-BG.js","static/lib/datejs/globalization/en-GB.js","static/lib/datejs/globalization/smj-NO.js","static/lib/datejs/globalization/nn-NO.js","static/lib/datejs/globalization/sms-FI.js","static/lib/datejs/globalization/ar-TN.js","static/lib/datejs/globalization/es-PY.js","static/lib/datejs/globalization/zh-TW.js","static/lib/datejs/globalization/ar-EG.js","static/lib/datejs/globalization/tt-RU.js","static/lib/datejs/globalization/en-CA.js","static/lib/datejs/globalization/hu-HU.js","static/lib/datejs/globalization/es-AR.js","static/lib/datejs/globalization/de-AT.js","static/lib/datejs/globalization/sw-KE.js","static/lib/datejs/globalization/fr-CA.js","static/lib/datejs/globalization/it-CH.js","static/lib/datejs/globalization/iu-CA.js","static/lib/datejs/globalization/ar-SA.js","static/lib/datejs/globalization/fr-MC.js","static/lib/datejs/globalization/xh-ZA.js","static/lib/datejs/globalization/sr-Cyrl-CS.js","static/lib/datejs/globalization/tlh-TLH.js","static/lib/datejs/globalization/ml-IN.js","static/lib/datejs/globalization/sr-Cyrl-BA.js","static/lib/datejs/globalization/mr-IN.js","static/lib/datejs/globalization/en-PH.js","static/lib/datejs/globalization/hi-IN.js","static/lib/datejs/globalization/es-MX.js","static/lib/datejs/globalization/oc-FR.js","static/lib/datejs/globalization/ar-KW.js","static/lib/datejs/globalization/pt-PT.js","static/lib/datejs/globalization/sr-RS.js","static/lib/datejs/globalization/ky-KG.js","static/lib/datejs/globalization/fr-LU.js","static/lib/datejs/globalization/pt-BR.js","static/lib/datejs/globalization/ar-YE.js","static/lib/datejs/globalization/fi-FI.js","static/lib/datejs/globalization/eu-ES.js","static/lib/datejs/globalization/hy-AM.js","static/lib/datejs/globalization/gl-ES.js","static/lib/datejs/globalization/ab-RU.js","static/lib/datejs/globalization/fa-IR.js","static/lib/datejs/globalization/quz-BO.js","static/lib/datejs/globalization/en-BZ.js","static/lib/datejs/globalization/fr-CH.js","static/lib/datejs/globalization/ka-GE.js","static/lib/datejs/globalization/ns-ZA.js","static/lib/datejs/globalization/kn-IN.js","static/lib/datejs/globalization/kk-KZ.js","static/lib/datejs/globalization/kok-IN.js","static/lib/datejs/globalization/en-AU.js","static/lib/datejs/globalization/en-ZW.js","static/lib/datejs/globalization/is-IS.js","static/lib/datejs/globalization/de-LU.js","static/lib/datejs/globalization/hr-HR.js","static/lib/datejs/globalization/se-SE.js","static/lib/datejs/globalization/es-DO.js","static/lib/datejs/globalization/tn-ZA.js","static/lib/datejs/globalization/en-JM.js","static/lib/datejs/globalization/es-BO.js","static/lib/datejs/globalization/ar-AR.js","static/lib/datejs/globalization/zh-HK.js","static/lib/datejs/globalization/nl-BE.js","static/lib/datejs/globalization/mt-MT.js","static/lib/datejs/globalization/ro-RO.js","static/lib/datejs/globalization/ko-KP.js","static/lib/datejs/globalization/ja-JP.js","static/lib/datejs/globalization/cy-GB.js","static/lib/datejs/globalization/ar-BH.js","static/lib/datejs/globalization/es-GT.js","static/lib/datejs/globalization/zh-MO.js","static/lib/datejs/globalization/lt-LT.js","static/lib/datejs/globalization/sr@latin.js","static/lib/datejs/globalization/tr-TR.js","static/lib/datejs/globalization/syr-SY.js","static/lib/datejs/globalization/ar-AE.js","static/lib/datejs/globalization/ar-SY.js","static/lib/datejs/globalization/ms-BN.js","static/lib/datejs/globalization/dv-MV.js","static/lib/datejs/globalization/fr-FR.js","static/lib/datejs/globalization/it-IT.js","static/lib/datejs/globalization/en-US.js","static/lib/datejs/globalization/es-CO.js","static/lib/datejs/globalization/es-HN.js","static/lib/datejs/globalization/ar-LY.js","static/lib/datejs/globalization/fo-FO.js","static/lib/datejs/globalization/sk-SK.js","static/lib/datejs/globalization/es-SV.js","static/lib/datejs/globalization/sq-AL.js","static/lib/datejs/globalization/quz-EC.js","static/lib/datejs/globalization/ta-IN.js","static/lib/datejs/globalization/ar-MA.js","static/lib/datejs/globalization/uz-Cyrl-UZ.js","static/lib/datejs/globalization/en-TT.js","static/lib/datejs/globalization/ar-OM.js","static/lib/datejs/globalization/ar-QA.js","static/lib/datejs/globalization/ar-JO.js","static/lib/datejs/globalization/az-Latn-AZ.js","static/lib/datejs/globalization/sma-SE.js","static/lib/datejs/globalization/smn-FI.js","static/lib/datejs/globalization/se-FI.js","static/lib/datejs/globalization/fr-BE.js","static/lib/datejs/globalization/mi-NZ.js","static/lib/datejs/globalization/si-LK.js","static/lib/datejs/globalization/es-PE.js","static/lib/datejs/globalization/uk-UA.js","static/lib/datejs/globalization/es-CL.js","static/lib/datejs/globalization/ar-IQ.js","static/lib/datejs/globalization/sr-Latn-BA.js","static/lib/datejs/globalization/en-IE.js","static/lib/datejs/globalization/en-029.js","static/lib/datejs/globalization/en-NZ.js","static/lib/datejs/globalization/lv-LV.js","static/lib/datejs/globalization/cs-CZ.js","static/lib/datejs/globalization/vi-VN.js","static/lib/datejs/globalization/es-EC.js","static/lib/datejs/globalization/mk-MK.js","static/lib/datejs/globalization/da-DK.js","static/lib/datejs/globalization/bs-BS.js","static/lib/datejs/globalization/ar-LB.js","static/lib/datejs/globalization/sv-FI.js","static/lib/datejs/globalization/sl-SI.js","static/lib/datejs/globalization/nb-NO.js","static/lib/datejs/globalization/id-ID.js","static/lib/datejs/globalization/hr-BA.js","static/lib/datejs/globalization/az-Cyrl-AZ.js","static/lib/datejs/globalization/sa-IN.js","static/lib/datejs/globalization/th-TH.js","static/lib/datejs/globalization/ar-DZ.js","static/lib/datejs/globalization/et-EE.js","static/lib/datejs/globalization/nl-NL.js","static/lib/datejs/globalization/ms-MY.js","static/lib/datejs/globalization/quz-PE.js","static/lib/datejs/globalization/de-CH.js","static/lib/datejs/globalization/es-NI.js","static/lib/datejs/globalization/es-ES.js","static/lib/datejs/globalization/gu-IN.js","static/lib/datejs/globalization/ru-RU.js","static/lib/datejs/globalization/sr-Latn-CS.js","static/lib/datejs/globalization/de-DE.js","static/lib/datejs/globalization/sv-SE.js","static/lib/datejs/globalization/zu-ZA.js","static/lib/datejs/globalization/es-VE.js","static/lib/datejs/globalization/sma-NO.js","static/lib/datejs/globalization/es-CR.js","static/lib/datejs/globalization/be-BY.js","static/lib/datejs/globalization/bs-Latn-BA.js","static/lib/datejs/globalization/he-IL.js","static/lib/datejs/globalization/te-IN.js","static/lib/datejs/globalization/es-PA.js","static/lib/datejs/globalization/pl-PL.js","static/lib/datejs/globalization/uz-Latn-UZ.js","static/lib/datejs/globalization/zh-SG.js","static/lib/datejs/globalization/af-ZA.js","static/lib/datejs/globalization/es-UY.js","static/lib/datejs/globalization/ca-ES.js","static/lib/datejs/globalization/smj-SE.js","static/lib/datejs/globalization/el-GR.js","static/lib/datejs/globalization/pa-IN.js","static/lib/datejs/globalization/am-ET.js","static/lib/datejs/globalization/en-ZA.js","static/lib/datejs/globalization/mn-MN.js","static/lib/datejs/globalization/es-PR.js","static/lib/datejs/globalization/ur-PK.js","static/lib/datejs/globalization/ko-KR.js","static/lib/datejs/globalization/se-NO.js","static/lib/datejs/globalization/de-LI.js","static/lib/jquery.validate/jquery.validate.js","static/lib/jquery.textext/jquery.textext.js","static/lib/jquery.scrollTo/jquery.scrollTo.js","static/lib/jquery.scrollTo/jquery.scrollTo-min.js","static/lib/jquery.ui/css/smoothness/images/ui-icons_cd0a0a_256x240.png","static/lib/jquery.ui/css/smoothness/images/ui-bg_glass_75_dadada_1x400.png","static/lib/jquery.ui/css/smoothness/images/ui-bg_glass_55_fbf9ee_1x400.png","static/lib/jquery.ui/css/smoothness/images/ui-icons_2e83ff_256x240.png","static/lib/jquery.ui/css/smoothness/images/ui-icons_222222_256x240.png","static/lib/jquery.ui/css/smoothness/images/ui-bg_highlight-soft_75_cccccc_1x100.png","static/lib/jquery.ui/css/smoothness/images/ui-bg_glass_95_fef1ec_1x400.png","static/lib/jquery.ui/css/smoothness/images/ui-bg_flat_75_ffffff_40x100.png","static/lib/jquery.ui/css/smoothness/images/ui-bg_glass_65_ffffff_1x400.png","static/lib/jquery.ui/css/smoothness/images/ui-icons_454545_256x240.png","static/lib/jquery.ui/css/smoothness/images/ui-icons_888888_256x240.png","static/lib/jquery.ui/css/smoothness/images/ui-bg_flat_0_aaaaaa_40x100.png","static/lib/jquery.ui/css/smoothness/images/ui-bg_glass_75_e6e6e6_1x400.png","static/lib/jquery.ui/js/jquery-ui-1.9.1.custom.js","static/lib/jquery.timeago/jquery.timeago.js","static/lib/backbone/backbone.js","static/lib/jquery.ui.bootstrap/bootstrap/js/bootstrap-alerts.js","static/lib/jquery.ui.bootstrap/bootstrap/js/bootstrap-scrollspy.js","static/lib/jquery.ui.bootstrap/bootstrap/js/bootstrap-tabs.js","static/lib/jquery.ui.bootstrap/bootstrap/js/bootstrap-modal.js","static/lib/jquery.ui.bootstrap/bootstrap/js/bootstrap-dropdown.js","static/lib/jquery.ui.bootstrap/bootstrap/js/bootstrap-buttons.js","static/lib/jquery.ui.bootstrap/bootstrap/js/bootstrap-twipsy.js","static/lib/jquery.ui.bootstrap/bootstrap/js/bootstrap-popover.js","static/lib/jquery.ui.bootstrap/third-party/jQuery-UI-Date-Range-Picker/index.html","static/lib/jquery.ui.bootstrap/third-party/jQuery-UI-Date-Range-Picker/js/daterangepicker.jQuery.js","static/lib/jquery.ui.bootstrap/third-party/jQuery-UI-Date-Range-Picker/js/daterangepicker.jQuery.compressed.js","static/lib/jquery.ui.bootstrap/third-party/jQuery-UI-Date-Range-Picker/js/date.js","static/lib/jquery.ui.bootstrap/third-party/wijmo/jquery.wijmo-open.1.5.0.min.js","static/lib/jquery.ui.bootstrap/third-party/wijmo/jquery.bgiframe-2.1.3-pre.js","static/lib/jquery.ui.bootstrap/third-party/wijmo/jquery.mousewheel.min.js","static/lib/jquery.ui.bootstrap/third-party/jQuery-UI-FileInput/images/icon-media.gif","static/lib/jquery.ui.bootstrap/third-party/jQuery-UI-FileInput/images/icon-zip.gif","static/lib/jquery.ui.bootstrap/third-party/jQuery-UI-FileInput/images/icon-image.gif","static/lib/jquery.ui.bootstrap/third-party/jQuery-UI-FileInput/images/icon-generic.gif","static/lib/jquery.ui.bootstrap/third-party/jQuery-UI-FileInput/js/fileinput.jquery.js","static/lib/jquery.ui.bootstrap/third-party/jQuery-UI-FileInput/js/enhance.min.js","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-icons_cd0a0a_256x240.png","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-bg_glass_75_dadada_1x400.png","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-bg_glass_55_fbf9ee_1x400.png","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-icons_2e83ff_256x240.png","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-icons_222222_256x240.png","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-bg_highlight-soft_75_cccccc_1x100.png","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-bg_inset-soft_95_fef1ec_1x100.png","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-bg_glass_75_ffffff_1x400.png","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-bg_glass_65_ffffff_1x400.png","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-icons_454545_256x240.png","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-icons_f6cf3b_256x240.png","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-icons_888888_256x240.png","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-bg_flat_0_aaaaaa_40x100.png","static/lib/jquery.ui.bootstrap/css/custom-theme/images/ui-bg_glass_75_e6e6e6_1x400.png","static/lib/qunit/qunit.js","static/lib/jquery.MD5/jquery.md5.js","static/lib/jquery.MD5/tests/index.html","static/lib/jquery.MD5/tests/tests.js","static/lib/qweb/qweb-test-attributes.xml","static/lib/qweb/qweb.js","static/lib/qweb/qweb-benchmark.xml","static/lib/qweb/qweb-test-foreach.xml","static/lib/qweb/qweb-test-set.xml","static/lib/qweb/qweb-test-call.xml","static/lib/qweb/qweb-test-extend.xml","static/lib/qweb/qweb-test.js.html","static/lib/qweb/qweb-test-output.xml","static/lib/qweb/qweb-test-conditionals.xml","static/lib/qweb/qweb2.js","static/lib/qweb/qweb-benchmark.html","static/lib/jquery.tipsy/jquery.tipsy.js","static/lib/spinjs/spin.js","static/lib/jquery.blockUI/jquery.blockUI.js","static/lib/underscore/underscore.js","static/lib/underscore/underscore.string.js","static/lib/underscore/underscore.string.min.js","static/lib/underscore/underscore-min.js","static/lib/py.js/lib/py.js","static/lib/jquery/jquery-1.8.3.js","static/src/fixbind.js","static/src/xml/base.xml","static/src/font/mnmliconsv21-webfont.woff","static/src/font/entypo-webfont.woff","static/src/font/entypo-webfont.eot","static/src/font/mnmliconsv21-webfont.ttf","static/src/font/entypo-webfont.ttf","static/src/font/mnmliconsv21-webfont.eot","static/src/font/mnmliconsv21-webfont.svg","static/src/font/entypo-webfont.svg","static/src/js/testing.js","static/src/js/data_export.js","static/src/js/pyeval.js","static/src/js/view_list.js","static/src/js/view_tree.js","static/src/js/dates.js","static/src/js/data.js","static/src/js/view_list_editable.js","static/src/js/views.js","static/src/js/test_support.js","static/src/js/coresetup.js","static/src/js/formats.js","static/src/js/boot.js","static/src/js/chrome.js","static/src/js/corelib.js","static/src/js/search.js","static/src/js/view_form.js","static/src/img/db.png","static/src/img/stock_person.png","static/src/img/gradientlinebg.gif","static/src/img/pencil.gif","static/src/img/product.png","static/src/img/pager_last.png","static/src/img/iconset-a-help.png","static/src/img/collapse.gif","static/src/img/toggle-a-bg.png","static/src/img/partner.png","static/src/img/expand.gif","static/src/img/down-arrow.png","static/src/img/favicon.ico","static/src/img/throbber.gif","static/src/img/form_steps.png","static/src/img/back-enable.jpg","static/src/img/iconset-d-drop.gif","static/src/img/views-icons-a.png","static/src/img/pager_first.png","static/src/img/search_reset.gif","static/src/img/graph_background.png","static/src/img/logo.png","static/src/img/user_menu_avatar.png","static/src/img/logo2.png","static/src/img/form_sheetbg.png","static/src/img/view_empty_arrow.png","static/src/img/attachments-close.png","static/src/img/add-shortcut.png","static/src/img/pager_next.png","static/src/img/iconset-a-help.gif","static/src/img/remove-shortcut.png","static/src/img/box-a-header-a.gif","static/src/img/accessories-archiver.png","static/src/img/placeholder.png","static/src/img/nologo.png","static/src/img/warning.png","static/src/img/header.gif","static/src/img/pager_previous.png","static/src/img/iconset-b-remove.png","static/src/img/sep-a.gif","static/src/img/icons/STOCK_INDENT.png","static/src/img/icons/gtk-cut.png","static/src/img/icons/STOCK_MEDIA_FORWARD.png","static/src/img/icons/gtk-find-and-replace.png","static/src/img/icons/gtk-sort-descending.png","static/src/img/icons/terp-folder-violet.png","static/src/img/icons/STOCK_APPLY.png","static/src/img/icons/stock_calendar.png","static/src/img/icons/STOCK_JUSTIFY_CENTER.png","static/src/img/icons/gtk-clear.png","static/src/img/icons/STOCK_UNDELETE.png","static/src/img/icons/STOCK_GOTO_BOTTOM.png","static/src/img/icons/STOCK_EDIT.png","static/src/img/icons/STOCK_CONNECT.png","static/src/img/icons/gtk-dnd-multiple.png","static/src/img/icons/gtk-select-font.png","static/src/img/icons/STOCK_JUSTIFY_FILL.png","static/src/img/icons/gtk-print.png","static/src/img/icons/STOCK_MEDIA_REWIND.png","static/src/img/icons/STOCK_DIRECTORY_MENU.png","static/src/img/icons/terp-rating-rated.png","static/src/img/icons/terp-dolar.png","static/src/img/icons/gtk-sort-ascending.png","static/src/img/icons/terp-folder-blue.png","static/src/img/icons/gtk-media-rewind.png","static/src/img/icons/stock_person.png","static/src/img/icons/STOCK_GO_UP.png","static/src/img/icons/star-off.png","static/src/img/icons/gtk-harddisk.png","static/src/img/icons/terp-stage.png","static/src/img/icons/STOCK_HARDDISK.png","static/src/img/icons/terp-mail-replied.png","static/src/img/icons/terp-personal+.png","static/src/img/icons/gtk-save-as.png","static/src/img/icons/STOCK_FILE.png","static/src/img/icons/terp-purchase.png","static/src/img/icons/gtk-quit.png","static/src/img/icons/STOCK_MEDIA_STOP.png","static/src/img/icons/terp-dolar_ok!.png","static/src/img/icons/STOCK_SAVE.png","static/src/img/icons/terp-report.png","static/src/img/icons/gtk-new.png","static/src/img/icons/terp-document-new.png","static/src/img/icons/terp-crm.png","static/src/img/icons/gtk-go-down.png","static/src/img/icons/terp-hr.png","static/src/img/icons/gtk-goto-last.png","static/src/img/icons/gtk-about.png","static/src/img/icons/STOCK_REMOVE.png","static/src/img/icons/gtk-zoom-in.png","static/src/img/icons/gtk-goto-first.png","static/src/img/icons/STOCK_SPELL_CHECK.png","static/src/img/icons/STOCK_FULLSCREEN.png","static/src/img/icons/terp-mail-.png","static/src/img/icons/STOCK_UNDERLINE.png","static/src/img/icons/terp-stock_effects-object-colorize.png","static/src/img/icons/STOCK_DIALOG_INFO.png","static/src/img/icons/gtk-redo.png","static/src/img/icons/gtk-convert.png","static/src/img/icons/gtk-preferences.png","static/src/img/icons/STOCK_STOP.png","static/src/img/icons/STOCK_INDEX.png","static/src/img/icons/gtk-justify-left.png","static/src/img/icons/terp-emblem-important.png","static/src/img/icons/STOCK_SAVE_AS.png","static/src/img/icons/gtk-media-previous.png","static/src/img/icons/gtk-no.png","static/src/img/icons/gtk-goto-bottom.png","static/src/img/icons/gtk-ok.png","static/src/img/icons/kanban-pause.png","static/src/img/icons/terp-stock_zoom.png","static/src/img/icons/terp-account.png","static/src/img/icons/gtk-media-forward.png","static/src/img/icons/STOCK_MISSING_IMAGE.png","static/src/img/icons/terp-locked.png","static/src/img/icons/STOCK_CANCEL.png","static/src/img/icons/terp-partner.png","static/src/img/icons/STOCK_CUT.png","static/src/img/icons/terp-gtk-media-pause.png","static/src/img/icons/gtk-paste.png","static/src/img/icons/gtk-dialog-authentication.png","static/src/img/icons/STOCK_REDO.png","static/src/img/icons/STOCK_PREFERENCES.png","static/src/img/icons/gtk-connect.png","static/src/img/icons/index2.jpeg","static/src/img/icons/terp-mail_delete.png","static/src/img/icons/terp-calendar.png","static/src/img/icons/gtk-leave-fullscreen.png","static/src/img/icons/STOCK_NO.png","static/src/img/icons/STOCK_PASTE.png","static/src/img/icons/gtk-copy.png","static/src/img/icons/terp-product.png","static/src/img/icons/STOCK_MEDIA_PREVIOUS.png","static/src/img/icons/STOCK_FIND_AND_REPLACE.png","static/src/img/icons/gtk-zoom-100.png","static/src/img/icons/gtk-info.png","static/src/img/icons/STOCK_PRINT.png","static/src/img/icons/terp-mail-forward.png","static/src/img/icons/STOCK_SORT_ASCENDING.png","static/src/img/icons/STOCK_DIALOG_QUESTION.png","static/src/img/icons/index1.jpeg","static/src/img/icons/gtk-print-preview.png","static/src/img/icons/terp-graph.png","static/src/img/icons/gtk-indent.png","static/src/img/icons/STOCK_MEDIA_RECORD.png","static/src/img/icons/gtk-file.png","static/src/img/icons/terp-gtk-jump-to-rtl.png","static/src/img/icons/STOCK_MEDIA_NEXT.png","static/src/img/icons/terp-stock_format-default.png","static/src/img/icons/gtk-directory.png","static/src/img/icons/gtk-fullscreen.png","static/src/img/icons/gtk-save.png","static/src/img/icons/STOCK_PROPERTIES.png","static/src/img/icons/STOCK_GOTO_FIRST.png","static/src/img/icons/STOCK_INFO.png","static/src/img/icons/terp-personal.png","static/src/img/icons/gtk-revert-to-saved.png","static/src/img/icons/terp-folder-orange.png","static/src/img/icons/stock.png","static/src/img/icons/gtk-directory-remote.png","static/src/img/icons/STOCK_CLEAR.png","static/src/img/icons/gtk-media-next.png","static/src/img/icons/terp-stock_format-scientific.png","static/src/img/icons/STOCK_DIALOG_AUTHENTICATION.png","static/src/img/icons/terp-dialog-close.png","static/src/img/icons/gtk-network.png","static/src/img/icons/terp-mrp.png","static/src/img/icons/gtk-stop.png","static/src/img/icons/STOCK_ZOOM_OUT.png","static/src/img/icons/gtk-go-up.png","static/src/img/icons/STOCK_GO_DOWN.png","static/src/img/icons/terp-marketing.png","static/src/img/icons/STOCK_DISCONNECT.png","static/src/img/icons/STOCK_OK.png","static/src/img/icons/terp-check.png","static/src/img/icons/gtk-zoom-fit.png","static/src/img/icons/STOCK_CONVERT.png","static/src/img/icons/stock_translate.png","static/src/img/icons/gtk-justify-right.png","static/src/img/icons/STOCK_SELECT_FONT.png","static/src/img/icons/STOCK_ITALIC.png","static/src/img/icons/gtk-delete.png","static/src/img/icons/terp-tools.png","static/src/img/icons/gtk-add.png","static/src/img/icons/STOCK_COPY.png","static/src/img/icons/STOCK_ADD.png","static/src/img/icons/gtk-media-record.png","static/src/img/icons/gtk-close.png","static/src/img/icons/gtk-media-stop.png","static/src/img/icons/save-document.png","static/src/img/icons/terp-translate.png","static/src/img/icons/gtk-underline.png","static/src/img/icons/kanban-stop.png","static/src/img/icons/terp-mail-message-new.png","static/src/img/icons/STOCK_GO_BACK.png","static/src/img/icons/STOCK_GO_FORWARD.png","static/src/img/icons/gtk-edit.png","static/src/img/icons/gtk-apply.png","static/src/img/icons/STOCK_PRINT_PREVIEW.png","static/src/img/icons/gtk-cdrom.png","static/src/img/icons/color-picker.png","static/src/img/icons/terp-call-start.png","static/src/img/icons/STOCK_COLOR_PICKER.png","static/src/img/icons/STOCK_GOTO_TOP.png","static/src/img/icons/STOCK_CDROM.png","static/src/img/icons/terp-gtk-select-all.png","static/src/img/icons/terp-face-plain.png","static/src/img/icons/STOCK_MEDIA_PAUSE.png","static/src/img/icons/gtk-help.png","static/src/img/icons/gtk-open.png","static/src/img/icons/gtk-dialog-info.png","static/src/img/icons/terp-go-month.png","static/src/img/icons/gtk-color-picker.png","static/src/img/icons/STOCK_ZOOM_FIT.png","static/src/img/icons/terp-stock_align_left_24.png","static/src/img/icons/STOCK_ZOOM_IN.png","static/src/img/icons/terp-personal-.png","static/src/img/icons/terp-gdu-smart-failing.png","static/src/img/icons/gtk-unindent.png","static/src/img/icons/terp-gtk-go-back-ltr.png","static/src/img/icons/terp-go-home.png","static/src/img/icons/stock_log.png","static/src/img/icons/terp-sale.png","static/src/img/icons/STOCK_GOTO_LAST.png","static/src/img/icons/terp-accessories-archiver.png","static/src/img/icons/STOCK_NETWORK.png","static/src/img/icons/terp-project.png","static/src/img/icons/STOCK_HOME.png","static/src/img/icons/terp-folder-yellow.png","static/src/img/icons/gtk-italic.png","static/src/img/icons/STOCK_UNDO.png","static/src/img/icons/gtk-go-back.png","static/src/img/icons/STOCK_MEDIA_PLAY.png","static/src/img/icons/gtk-select-color.png","static/src/img/icons/gtk-zoom-out.png","static/src/img/icons/gtk-home.png","static/src/img/icons/STOCK_STRIKETHROUGH.png","static/src/img/icons/gtk-find.png","static/src/img/icons/gtk-refresh.png","static/src/img/icons/gtk-dialog-warning.png","static/src/img/icons/terp-gtk-jump-to-ltr.png","static/src/img/icons/gtk-media-pause.png","static/src/img/icons/STOCK_BOLD.png","static/src/img/icons/gtk-justify-center.png","static/src/img/icons/terp-idea.png","static/src/img/icons/STOCK_CLOSE.png","static/src/img/icons/gtk-floppy.png","static/src/img/icons/terp-gnome-cpu-frequency-applet+.png","static/src/img/icons/terp-accessories-archiver-minus.png","static/src/img/icons/STOCK_SELECT_COLOR.png","static/src/img/icons/STOCK_REVERT_TO_SAVED.png","static/src/img/icons/gtk-properties.png","static/src/img/icons/gtk-bold.png","static/src/img/icons/STOCK_DIALOG_WARNING.png","static/src/img/icons/STOCK_EXECUTE.png","static/src/img/icons/STOCK_DND_MULTIPLE.png","static/src/img/icons/STOCK_QUIT.png","static/src/img/icons/terp-camera_test.png","static/src/img/icons/STOCK_FLOPPY.png","static/src/img/icons/STOCK_DIRECTORY.png","static/src/img/icons/STOCK_DND.png","static/src/img/icons/STOCK_HELP.png","static/src/img/icons/gtk-cancel.png","static/src/img/icons/STOCK_NEW.png","static/src/img/icons/gtk-goto-top.png","static/src/img/icons/STOCK_YES.png","static/src/img/icons/STOCK_JUSTIFY_LEFT.png","static/src/img/icons/terp-accessories-archiver+.png","static/src/img/icons/STOCK_DELETE.png","static/src/img/icons/STOCK_LEAVE_FULLSCREEN.png","static/src/img/icons/gtk-execute.png","static/src/img/icons/kanban-apply.png","static/src/img/icons/STOCK_UNINDENT.png","static/src/img/icons/stock_graph.png","static/src/img/icons/gtk-paste-v.png","static/src/img/icons/terp-go-today.png","static/src/img/icons/terp-folder-green.png","static/src/img/icons/gtk-dialog-error.png","static/src/img/icons/gtk-undelete.png","static/src/img/icons/terp-stock.png","static/src/img/icons/gtk-spell-check.png","static/src/img/icons/gtk-dialog-question.png","static/src/img/icons/gtk-justify-fill.png","static/src/img/icons/terp-administration.png","static/src/img/icons/gtk-dnd.png","static/src/img/icons/STOCK_ABOUT.png","static/src/img/icons/terp-gtk-stop.png","static/src/img/icons/STOCK_REFRESH.png","static/src/img/icons/terp-go-year.png","static/src/img/icons/gtk-jump-to.png","static/src/img/icons/gtk-index.png","static/src/img/icons/gtk-disconnect.png","static/src/img/icons/terp-gtk-go-back-rtl.png","static/src/img/icons/STOCK_JUSTIFY_RIGHT.png","static/src/img/icons/STOCK_OPEN.png","static/src/img/icons/gtk-remove.png","static/src/img/icons/gtk-go-forward.png","static/src/img/icons/gtk-undo.png","static/src/img/icons/STOCK_FIND.png","static/src/img/icons/gtk-strikethrough.png","static/src/img/icons/star-on.png","static/src/img/icons/gtk-media-play.png","static/src/img/icons/STOCK_DIALOG_ERROR.png","static/src/img/icons/gtk-missing-image.png","static/src/img/icons/terp-go-week.png","static/src/img/icons/STOCK_JUMP_TO.png","static/src/img/icons/STOCK_ZOOM_100.png","static/src/img/icons/gtk-yes.png","static/src/img/icons/stock_gantt.png","static/src/img/icons/STOCK_SORT_DESCENDING.png","static/src/img/icons/terp-stock_symbol-selection.png","static/src/img/ui/group-expanded.png","static/src/img/ui/field_calendar.png","static/src/img/ui/group-folded.png","i18n/sk.po","i18n/fa.po","i18n/id.po","i18n/hr.po","i18n/da.po","i18n/sl.po","i18n/es_MX.po","i18n/fr_CA.po","i18n/ru.po","i18n/zh_TW.po","i18n/en_GB.po","i18n/cs.po","i18n/sq.po","i18n/fi.po","i18n/bn.po","i18n/pt_BR.po","i18n/ko.po","i18n/gu.po","i18n/bg.po","i18n/es_PE.po","i18n/ja.po","i18n/ro.po","i18n/hi.po","i18n/sr@latin.po","i18n/es.po","i18n/ka.po","i18n/mn.po","i18n/es_CL.po","i18n/it.po","i18n/nl.po","i18n/sv.po","i18n/es_EC.po","i18n/uk.po","i18n/bs.po","i18n/tr.po","i18n/en_AU.po","i18n/de.po","i18n/et.po","i18n/lo.po","i18n/lt.po","i18n/hu.po","i18n/vi.po","i18n/eu.po","i18n/gl.po","i18n/lv.po","i18n/mk.po","i18n/ca.po","i18n/fr.po","i18n/zh_CN.po","i18n/pl.po","i18n/he.po","i18n/th.po","i18n/nl_BE.po","i18n/es_DO.po","i18n/nb.po","i18n/ar.po","i18n/pt.po","i18n/es_CR.po","doc/images/tests.png","doc/images/db-query.png","doc/images/runner2.png","doc/images/runner.png","doc/images/tests3.png","doc/images/tests2.png","doc/_templates/sidebarlogo.html","doc/_templates/sidebarintro.html","doc/module/web_example.xml","doc/module/static/src/xml/web_example.xml","doc/module/static/src/js/first_module.js","doc/_themes/flask_small/layout.html","doc/_themes/flask/relations.html","doc/_themes/flask/layout.html","doc/_static/openerp.png","tests/qunitsuite/grunt/bootstrap.js","tests/qunitsuite/grunt/qunit-phantomjs-bridge.js","static/src/xml/*.xml","static/lib/jquery.ui.bootstrap/css/custom-theme/jquery-ui-1.9.0.custom.css","static/lib/jquery.ui.timepicker/css/jquery-ui-timepicker-addon.css","static/lib/jquery.ui.notify/css/ui.notify.css","static/lib/jquery.tipsy/tipsy.css","static/lib/jquery.textext/jquery.textext.css","static/src/css/base.css","static/src/css/data_export.css","static/lib/cleditor/jquery.cleditor.css","static/src/fixbind.js","static/lib/datejs/globalization/en-US.js","static/lib/datejs/core.js","static/lib/datejs/parser.js","static/lib/datejs/sugarpak.js","static/lib/datejs/extras.js","static/lib/jquery/jquery-1.8.3.js","static/lib/jquery.MD5/jquery.md5.js","static/lib/jquery.form/jquery.form.js","static/lib/jquery.validate/jquery.validate.js","static/lib/jquery.ba-bbq/jquery.ba-bbq.js","static/lib/spinjs/spin.js","static/lib/jquery.autosize/jquery.autosize.js","static/lib/jquery.blockUI/jquery.blockUI.js","static/lib/jquery.placeholder/jquery.placeholder.js","static/lib/jquery.ui/js/jquery-ui-1.9.1.custom.js","static/lib/jquery.ui.timepicker/js/jquery-ui-timepicker-addon.js","static/lib/jquery.ui.notify/js/jquery.notify.js","static/lib/jquery.deferred-queue/jquery.deferred-queue.js","static/lib/jquery.scrollTo/jquery.scrollTo-min.js","static/lib/jquery.tipsy/jquery.tipsy.js","static/lib/jquery.textext/jquery.textext.js","static/lib/jquery.timeago/jquery.timeago.js","static/lib/qweb/qweb2.js","static/lib/underscore/underscore.js","static/lib/underscore/underscore.string.js","static/lib/backbone/backbone.js","static/lib/cleditor/jquery.cleditor.js","static/lib/py.js/lib/py.js","static/src/js/boot.js","static/src/js/testing.js","static/src/js/pyeval.js","static/src/js/corelib.js","static/src/js/coresetup.js","static/src/js/dates.js","static/src/js/formats.js","static/src/js/chrome.js","static/src/js/views.js","static/src/js/data.js","static/src/js/data_export.js","static/src/js/search.js","static/src/js/view_form.js","static/src/js/view_list.js","static/src/js/view_list_editable.js","static/src/js/view_tree.js"],
    },
    install_requires = ["mock"],
    tests_require = ["unittest2"],
)
